.str,.atv{color:#9daa7e}.kwd,.tag{color:#d5b57c}.com{color:#726d73}.typ,.atn,.dec{color:#dd7e5e}.lit{color:#fcf0a4}.pun,.opn,.clo{color:#a78353}.pln{color:#889dbc}pre.prettyprint{border:0;padding-bottom:0.75em;padding-top:0.75em}